@@ -907,29 +907,30 @@ InterpreterObject *newInterpreterObject(const char *name)
 
     /*
      * If running in daemon mode and a home directory was set then
-     * insert an empty string at the start of the Python module search
-     * path so the current working directory will be searched. This
-     * makes things similar to when using the Python interpreter on the
-     * command line. If the current working directory changes then where
-     * it looks follows, so doesn't always look in home.
+     * insert the home directory at the start of the Python module
+     * search path. This makes things similar to when using the Python
+     * interpreter on the command line with a script.
      */
 
 #if defined(MOD_WSGI_WITH_DAEMONS)
     if (wsgi_daemon_process && wsgi_daemon_process->group->home) {
         PyObject *path = NULL;
+        const char *home = wsgi_daemon_process->group->home;
 
         path = PySys_GetObject("path");
 
         if (module && path) {
-            PyObject *empty;
+            PyObject *item;
 
 #if PY_MAJOR_VERSION >= 3
-            empty = PyUnicode_DecodeLatin1("", strlen(""), NULL);
+            item = PyUnicode_Decode(home, strlen(home),
+                                    Py_FileSystemDefaultEncoding,
+                                    "surrogateescape");
 #else
-            empty = PyString_FromString("");
+            item = PyString_FromString(home);
 #endif
-            PyList_Insert(path, 0, empty);
-            Py_DECREF(empty);
+            PyList_Insert(path, 0, item);
+            Py_DECREF(item);
         }
 
         Py_XDECREF(module);
@@ -1168,7 +1169,11 @@ InterpreterObject *newInterpreterObject(const char *name)
     object = PyLong_FromLong(max_threads);
     PyModule_AddObject(module, "threads_per_process", object);
 
+#if AP_MODULE_MAGIC_AT_LEAST(20051115,4)
     str = ap_get_server_description();
+#else
+    str = ap_get_server_version();
+#endif
 #if PY_MAJOR_VERSION >= 3
     object = PyUnicode_DecodeLatin1(str, strlen(str), NULL);
 #else
@@ -1344,11 +1349,14 @@ InterpreterObject *newInterpreterObject(const char *name)
 static void Interpreter_dealloc(InterpreterObject *self)
 {
     PyThreadState *tstate = NULL;
-    PyObject *exitfunc = NULL;
     PyObject *module = NULL;
 
     PyThreadState *tstate_enter = NULL;
 
+#if PY_MAJOR_VERSION < 3 || (PY_MAJOR_VERSION == 3 && PY_MINOR_VERSION < 4)
+    PyObject *exitfunc = NULL;
+#endif
+
     /*
      * We should always enter here with the Python GIL
      * held and an active thread state. This should only
@@ -1927,6 +1935,8 @@ void wsgi_python_init(apr_pool_t *p)
 {
     const char *python_home = 0;
 
+    int is_pyvenv = 0;
+
     /* Perform initialisation if required. */
 
     if (!Py_IsInitialized()) {
@@ -1982,23 +1992,29 @@ void wsgi_python_init(apr_pool_t *p)
             }
         }
 
-        /* Check for Python HOME being overridden. */
+#if defined(WIN32)
+        /*
+         * Check for Python HOME being overridden. This is only being
+         * used on Windows for now. For UNIX systems we actually do
+         * a fiddle and work out where the Python executable would be
+         * and set its location instead. This is to get around some
+         * brokeness in pyvenv in Python 3.X. We don't know if that
+         * workaround works for Windows yet, but since not supporting
+         * Windows for mod_wsgi 4.X as yet, doesn't matter.
+         */
 
         python_home = wsgi_server_config->python_home;
 
-#if defined(MOD_WSGI_WITH_DAEMONS)
-        if (wsgi_daemon_process && wsgi_daemon_process->group->python_home)
-            python_home = wsgi_daemon_process->group->python_home;
-#endif
-
-#if PY_MAJOR_VERSION >= 3
         if (python_home) {
-            wchar_t *s = NULL;
-            int len = strlen(python_home)+1;
-
             ap_log_error(APLOG_MARK, APLOG_INFO, 0, wsgi_server,
                          "mod_wsgi (pid=%d): Python home %s.", getpid(),
                          python_home);
+        }
+
+        if (python_home) {
+#if PY_MAJOR_VERSION >= 3
+            wchar_t *s = NULL;
+            int len = strlen(python_home)+1;
 
             s = (wchar_t *)apr_palloc(p, len*sizeof(wchar_t));
 
@@ -2008,16 +2024,123 @@ void wsgi_python_init(apr_pool_t *p)
             mbstowcs(s, python_home, len);
 #endif
             Py_SetPythonHome(s);
-        }
 #else
+            Py_SetPythonHome((char *)python_home);
+#endif
+        }
+
+#else
+        /*
+         * Now for the UNIX version of the code to set the Python HOME.
+         * For this things are a mess. If using pyvenv with Python 3.3+
+         * then setting Python HOME doesn't work. For it we need to use
+         * Python executable location. Everything else seems to be cool
+         * with setting Python HOME. We therefore need to detect when we
+         * have a pyvenv by looking for the presence of pyvenv.cfg file.
+         * We can simply just set Python executable everywhere as that
+         * doesn't work with brew Python on MacOS X.
+         */
+
+        python_home = wsgi_server_config->python_home;
+
+#if defined(MOD_WSGI_WITH_DAEMONS)
+        if (wsgi_daemon_process && wsgi_daemon_process->group->python_home)
+            python_home = wsgi_daemon_process->group->python_home;
+#endif
+
         if (python_home) {
+            apr_status_t rv;
+            apr_finfo_t finfo; 
+
+            char *pyvenv_cfg;
+
+            const char *python_exe = 0;
+
+#if PY_MAJOR_VERSION >= 3
+            wchar_t *s = NULL;
+            int len = 0;
+#endif
+
+            /*
+             * Is common to see people set the directory to an incorrect
+             * location, including to a location within an inaccessible
+             * user home directory, or to the 'python' executable itself.
+             * Try and validate that the location is accessible and is a
+             * directory.
+             */
+
             ap_log_error(APLOG_MARK, APLOG_INFO, 0, wsgi_server,
                          "mod_wsgi (pid=%d): Python home %s.", getpid(),
                          python_home);
 
-            Py_SetPythonHome((char *)python_home);
-        }
+            rv = apr_stat(&finfo, python_home, APR_FINFO_NORM, p);
+
+            if (rv != APR_SUCCESS) {
+                ap_log_error(APLOG_MARK, APLOG_WARNING, rv, wsgi_server,
+                             "mod_wsgi (pid=%d): Unable to stat Python home "
+                             "%s. Python interpreter may not be able to be "
+                             "initialized correctly. Verify the supplied path "
+                             "and access permissions for whole of the path.",
+                             getpid(), python_home);
+            }
+            else {
+                if (finfo.filetype != APR_DIR) {
+                    ap_log_error(APLOG_MARK, APLOG_WARNING, rv, wsgi_server,
+                                 "mod_wsgi (pid=%d): Python home %s is not "
+                                 "a directory. Python interpreter may not "
+                                 "be able to be initialized correctly. "
+                                 "Verify the supplied path.", getpid(),
+                                 python_home);
+                }
+                else if (access(python_home, X_OK) == -1) {
+                    ap_log_error(APLOG_MARK, APLOG_WARNING, rv, wsgi_server,
+                                 "mod_wsgi (pid=%d): Python home %s is not "
+                                 "accessible. Python interpreter may not "
+                                 "be able to be initialized correctly. "
+                                 "Verify the supplied path and access "
+                                 "permissions on the directory.", getpid(),
+                                 python_home);
+                }
+            }
+
+            /* Now detect whether have a pyvenv with Python 3.3+. */
+
+            pyvenv_cfg = apr_pstrcat(p, python_home, "/pyvenv.cfg", NULL);
+
+            if (access(pyvenv_cfg, R_OK) == 0)
+                is_pyvenv = 1;
+
+            if (is_pyvenv) {
+                /*
+                 * Embedded support for pyvenv is broken so need to
+                 * set Python executable location and cannot set the
+                 * Python HOME as is more desirable.
+                 */
+
+                python_exe = apr_pstrcat(p, python_home, "/bin/python", NULL);
+#if PY_MAJOR_VERSION >= 3
+                len = strlen(python_exe)+1;
+                s = (wchar_t *)apr_palloc(p, len*sizeof(wchar_t));
+                mbstowcs(s, python_exe, len);
+
+                Py_SetProgramName(s);
+#else
+                Py_SetProgramName((char *)python_exe);
 #endif
+            }
+            else {
+#if PY_MAJOR_VERSION >= 3
+                len = strlen(python_home)+1;
+                s = (wchar_t *)apr_palloc(p, len*sizeof(wchar_t));
+                mbstowcs(s, python_home, len);
+
+                Py_SetPythonHome(s);
+#else
+                Py_SetPythonHome((char *)python_home);
+#endif
+            }
+#endif
+        }
 
         /*
          * Set environment variable PYTHONHASHSEED. We need to
